unit vb_pvflib;

interface

uses classes, SysUtils, winsock, dialogs;

const
     bitmask: Array[0..16] of integer = ($0000, $0001, $0003, $0007, $000f,
                                         $001f, $003f, $007f, $00ff, $01ff,
                                         $03ff, $07ff, $0fff, $1fff, $3fff,
                                         $7fff, $ffff);
     SND_FORMAT_MULAW_8    = 1;
     SND_FORMAT_LINEAR_8   = 2;
     SND_FORMAT_LINEAR_16  = 3;


Type
    TpvfWriteData = procedure(Stream: TMemoryStream; Data: integer) of Object;
    TpvfReadData  = function(Stream: TMemoryStream): integer of Object;

    state_t = record
     data:  integer;
     nleft: integer;
    End;

    TReadFunctions = class(TObject)
     private
      function read_int_from_ascii(Stream: TMemoryStream): integer;
     public
      function read_bits(Stream: TMemoryStream; nbits: integer; var state: state_t): integer;
      function read_bits_reverse(Stream: TMemoryStream; nbits: integer; var state: state_t): integer;
      function read_pvf_data_8_ascii(Stream: TMemoryStream): integer;
      function read_pvf_data_16_ascii(Stream: TMemoryStream): integer;
      function read_pvf_data_32_ascii(Stream: TMemoryStream): integer;
      function read_pvf_data_8(Stream: TMemoryStream): integer;
      function read_pvf_data_16(Stream: TMemoryStream): integer;
      function read_pvf_data_32(Stream: TMemoryStream): integer;
    End;

    TWriteProcedures = class(TObject)
     public
      procedure write_bits(Stream: TMemoryStream; nbits, data: integer; var state: state_t);
      procedure write_bits_reverse (Stream: TMemoryStream; nbits, data: integer; var state: state_t);
      procedure write_pvf_data_8_ascii(Stream: TMemoryStream; data: integer);
      procedure write_pvf_data_16_ascii(Stream: TMemoryStream; data: integer);
      procedure write_pvf_data_32_ascii(Stream: TMemoryStream; data: integer);
      procedure write_pvf_data_8(Stream: TMemoryStream; data: integer);
      procedure write_pvf_data_16(Stream: TMemoryStream; data: integer);
      procedure write_pvf_data_32(Stream: TMemoryStream; data: integer);
      procedure StreamWriteLn(Stream: TMemoryStream; data: integer);
    End;

    Tpvf_header = class(TPersistent)
     private
      Fascii: Boolean;
      Fchannels: integer;
      Fspeed:    integer;
      Fnbits:     integer;
      FReadFunctions: TReadFunctions;
      FWriteProcedures: TWriteProcedures;
      Fread_pvf_data: TpvfReadData;
      Fwrite_pvf_data: TpvfWriteData;

      function StreamReadLn(Stream: TMemoryStream): String;
     public
      property Ascii: Boolean read Fascii write Fascii;
      property Channels: integer read Fchannels write Fchannels;
      property Speed: integer read Fspeed write Fspeed;
      property nBits: integer read Fnbits write Fnbits;
      property Pread_pvf_data: TpvfReadData read Fread_pvf_data write Fread_pvf_data;
      property Pwrite_pvf_data: TpvfWriteData read Fwrite_pvf_data write Fwrite_pvf_data;
      property ReadFunctions: TReadFunctions read FReadFunctions;
      property WriteProcedures: TWriteProcedures read FWriteProcedures;
      constructor Create;
      destructor Destroy; override;
      procedure Assign(Source: TPersistent); override;
      function  read_pvf_header(Stream: TMemoryStream): Boolean;
      function  write_pvf_header(Stream: TMemoryStream): Boolean;
      function  read_pvf_data(Stream: TMemoryStream): integer;
      procedure write_pvf_data(Stream: TMemoryStream; Data: integer);
    End;



implementation

var
   bits_data_new,
   bits_data: integer;

{ Tpvf_header }

procedure Tpvf_header.Assign(Source: TPersistent);
begin
  if Source is Tpvf_header then
    with Source as Tpvf_header do
    begin
     Self.Ascii           := Ascii;
     Self.Channels        := Channels;
     Self.Speed           := Speed;
     Self.nBits           := nBits;
     Self.Pread_pvf_data  := Pread_pvf_data;
     Self.Pwrite_pvf_data := Pwrite_pvf_data;
    end
  else
    inherited;
end;

constructor Tpvf_header.Create;
begin
 Self.FReadFunctions   := TReadFunctions.Create;
 Self.FWriteProcedures := TWriteProcedures.Create;

 Self.Ascii    := False;
 Self.Channels := 1;
 Self.Speed    := 8000;
 Self.nBits    := 32;
 Self.Pread_pvf_data  := FReadFunctions.read_pvf_data_32;
 Self.Pwrite_pvf_data := FWriteProcedures.write_pvf_data_32;
end;

destructor Tpvf_header.Destroy;
begin
 Self.FReadFunctions.Free;
 Self.FWriteProcedures.Free;
 inherited;
end;

function Tpvf_header.read_pvf_data(Stream: TMemoryStream): integer;
begin
 result := -1;
 if Assigned(Fread_pvf_data) Then result := Fread_pvf_data(Stream);
end;

function Tpvf_header.StreamReadLn(Stream: TMemoryStream): String;
Var
   MyStr:  String;
   MyChar: Char;
Begin
 MyStr := '';
 Repeat
  Begin
   Stream.Read(MyChar,1);
   MyStr := MyStr + MyChar;
  End;
 Until MyChar = #10;
 result := Trim(MyStr);
End;

function Tpvf_header.read_pvf_header(Stream: TMemoryStream): Boolean;
Var
   Buffer: String;
   TmpStr: String;
begin
 Buffer := StreamReadLn(Stream);
 If not ((CompareStr(Buffer,'PVF1') = 0) or (CompareStr(Buffer,'PVF2') = 0)) Then
  Begin
   result := False;
   Exit;
  End;
 Self.Ascii := (CompareStr(Buffer,'PVF2') = 0);

 Buffer := StreamReadLn(Stream);

 TmpStr := Copy(Buffer,1,Pos(' ',Buffer)-1);
 Self.Channels := StrToInt(TmpStr);
 Delete(Buffer,1,Pos(' ',Buffer));

 TmpStr := Copy(Buffer,1,Pos(' ',Buffer)-1);
 Self.Speed := StrToInt(TmpStr);
 Delete(Buffer,1,Pos(' ',Buffer));

 Self.nBits := StrToInt(Trim(Buffer));

 if ((Self.Channels < 1) or (32 < Self.Channels)) Then
  Begin
   Result := False;
   Exit;
  End;

 if ((Self.Speed < 0) or (50000 < Self.Speed)) Then
  Begin
   Result := False;
   Exit;
  End;

 if ((Self.nBits <> 8) and (Self.nBits <> 16) and (Self.nBits <> 32)) Then
  Begin
   Result := False;
   Exit;
  End;

 if Self.Ascii Then
  Begin
   case Self.nBits of
      8:   Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_8_ascii;
      16:  Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_16_ascii;
      32:  Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_32_ascii;
      Else
       Begin
        Result := False;
        Exit;
       End;
   End;
  End
 Else
  Begin
   case Self.nBits of
      8:   Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_8;
      16:  Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_16;
      32:  Self.Pread_pvf_data := Self.ReadFunctions.read_pvf_data_32;
      Else
       Begin
        Result := False;
        Exit;
       End;
   End;
  End;
 Result := True;
end;

function Tpvf_header.write_pvf_header(Stream: TMemoryStream): Boolean;
Var
   Buffer: Array[0..80] of Char;
   Count: Integer;
begin
 if Self.Ascii Then
  Begin
   StrPCopy(Buffer,format('%s%s%d %d %d%s',['PVF2',#10,Self.Channels,Self.Speed,Self.nBits,#10]));
   case Self.nBits of
      8:   Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_8_ascii;
      16:  Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_16_ascii;
      32:  Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_32_ascii;
      Else
       Begin
        Result := False;
        Exit;
       End;
   End;
  End
 Else
  Begin
   StrPCopy(Buffer,format('%s%s%d %d %d%s',['PVF1',#10,Self.Channels,Self.Speed,Self.nBits,#10]));
   case Self.nBits of
      8:   Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_8;
      16:  Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_16;
      32:  Self.Pwrite_pvf_data := Self.FWriteProcedures.write_pvf_data_32;
      Else
       Begin
        Result := False;
        Exit;
       End;
   End;
  End;
 Count := 0;
 While Buffer[Count] <> #0 Do
  Begin
   Stream.Write(Buffer[Count],1);
   Inc(Count);
  End;
 Result := True;
end;

procedure Tpvf_header.write_pvf_data(Stream: TMemoryStream; Data: integer);
begin
 If Assigned(Fwrite_pvf_data) Then Fwrite_pvf_data(Stream,Data);
end;

{Functions}

function TReadFunctions.read_int_from_ascii(Stream: TMemoryStream): integer;
Var
   MyStr:  String;
   MyChar: Char;
Begin
 MyStr := '';
 Repeat
  Begin
   Stream.Read(MyChar,1);
   MyStr := MyStr + MyChar;
  End;
 Until MyChar = #10;

 MyStr := Trim(MyStr);
 result := StrToInt(MyStr);
End;

function TReadFunctions.read_bits(Stream: TMemoryStream; nbits: integer;
  var state: state_t): integer;
Var
   MyByte:    Byte;
begin
 While (state.nleft < nbits) do
  Begin
   Stream.Read(MyByte,1);
   bits_data_new := MyByte;
   state.data  := (state.data shl 8) or bits_data_new;
   state.nleft := state.nleft + 8;
  End;

 state.nleft := state.nleft - nbits;
 result := (state.data shr state.nleft) and bitmask[nbits];
end;

procedure TWriteProcedures.write_bits(Stream: TMemoryStream; nbits, data: integer;
  var state: state_t);
Var
   MyByte: Byte;
begin
 state.data  := (state.data shl nbits) or (data and bitmask[nbits]);
 state.nleft := state.nleft + nbits;

 While (state.nleft >= 8) Do
  Begin
   MyByte := (( state.data shr (state.nleft - 8)) and $ff);
   Stream.Write(MyByte,1);
   state.nleft := state.nleft - 8;
  End;
end;

function TReadFunctions.read_bits_reverse(Stream: TMemoryStream;
  nbits: integer; var state: state_t): integer;
Var
   MyByte:   Byte;
begin
 While (state.nleft < nbits) do
  Begin
   Stream.Read(MyByte,1);
   state.data  := (state.data or (MyByte shl state.nleft));
   state.nleft := state.nleft + 8;
  End;

 bits_data := state.data and bitmask[nbits];
 state.nleft := state.nleft - nbits;
 state.data  := state.data shr nbits;
 result := bits_data;
end;

procedure TWriteProcedures.write_bits_reverse(Stream: TMemoryStream;
  nbits, data: integer; var state: state_t);
Var
   MyByte: Byte;
begin
 state.data  := (state.data or ((data and bitmask[nbits]) shl state.nleft));
 state.nleft := state.nleft + nbits;

 While (state.nleft >= 8) do
  Begin
   MyByte := (state.data and $ff);
   Stream.Write(MyByte,1);
   state.nleft := state.nleft - 8;
   state.data  := state.data shr 8;
  End;
end;

function TReadFunctions.read_pvf_data_8_ascii(Stream: TMemoryStream): integer;
Var
   data_new: integer;
Begin
 data_new := read_int_from_ascii(Stream);
 if (data_new < -128) then data_new := ShortInt(data_new);
 if (data_new > 127)  then data_new := ShortInt(data_new);
 result := data_new shl 16;
End;

function TReadFunctions.read_pvf_data_16_ascii(Stream: TMemoryStream): integer;
Var
   data_new: integer;
Begin
 data_new := read_int_from_ascii(Stream);
 if (data_new < -32768) then data_new := SmallInt(data_new);
 if (data_new > 32767)  then data_new := SmallInt(data_new);

 result := data_new shl 8;
End;

function TReadFunctions.read_pvf_data_32_ascii(Stream: TMemoryStream): integer;
Begin
 result := read_int_from_ascii(Stream);
End;

function TReadFunctions.read_pvf_data_8(Stream: TMemoryStream): integer;
Var
   data_new: ShortInt;
Begin
// result := 0;
 Stream.Read(data_new,1);
 result := data_new shl 16;
End;

function TReadFunctions.read_pvf_data_16(Stream: TMemoryStream): integer;
Var
//   data_new: SmallInt;
   MyByte:   Byte;
Begin
 Stream.Read(MyByte,1);
 result := MyByte shl 8;
 Stream.Read(MyByte,1);
 result := result or MyByte;
 result := result shl 8;
End;

function TReadFunctions.read_pvf_data_32(Stream: TMemoryStream): integer;
Var
//   data_new: integer;
   MyByte:   Byte;
Begin
 Stream.Read(MyByte,1);
 result := MyByte shl 24;
 Stream.Read(MyByte,1);
 result := result or (MyByte shl 16);
 Stream.Read(MyByte,1);
 result := result or (MyByte shl 8);
 Stream.Read(MyByte,1);
 result := result or MyByte;
End;

procedure TWriteProcedures.write_pvf_data_8_ascii(Stream: TMemoryStream; data: integer);
Var
//   MyStr: String;
   MyShort: ShortInt;
Begin
 MyShort := data shr 16;
 StreamWriteLn(Stream,MyShort)
End;

procedure TWriteProcedures.write_pvf_data_16_ascii(Stream: TMemoryStream; data: integer);
Var
//   MyStr: String;
   MySmall: SmallInt;
Begin
 MySmall := data shr 8;
 StreamWriteLn(Stream,MySmall)
End;

procedure TWriteProcedures.write_pvf_data_32_ascii(Stream: TMemoryStream; data: integer);
//Var
//   MyStr: String;
Begin
 StreamWriteLn(Stream,data)
End;

procedure TWriteProcedures.write_pvf_data_8(Stream: TMemoryStream; data: integer);
Var
   MyShort: ShortInt;
//   MyChar:  Char;
Begin
 MyShort := data shr 16;
 Stream.Write(MyShort,1);
End;

procedure TWriteProcedures.write_pvf_data_16(Stream: TMemoryStream; data: integer);
Var
   MySmall: SmallInt;
Begin
 MySmall := data shr 8;
 MySmall := htons(word(MySmall));
 Stream.Write(MySmall,2);
End;

procedure TWriteProcedures.write_pvf_data_32(Stream: TMemoryStream; data: integer);
Var
   MyInt: integer;
Begin
 MyInt := htonl(data);
 Stream.Write(MyInt,4);
End;


procedure TWriteProcedures.StreamWriteLn(Stream: TMemoryStream; data: integer);
Var
   Buffer: Array[0..10] of Char;
   Count: Integer;
begin
 StrPCopy(Buffer,IntToStr(data)+#10);
 Count := 0;
 While Buffer[Count] <> #0 Do
  Begin
   Stream.Write(Buffer[Count],1);
   Inc(Count);
  End;
end;

end.

