unit VB_SoundConvert;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  vb_au, vb_pvflib, vb_wav, vb_zyxel;

type

  TVBHeader = packed record
    magic:       Array[0..3] of Char;
    time:        longword;
    compression: longword;
    callerid:    Array[0..32] of Char;
    name:        Array[0..32] of Char;
    phone:       Array[0..32] of Char;
    location:    Array[0..64] of Char;
    reserved1:   Array[0..32] of Char;
    reserved2:   Array[0..32] of Char;
    reserved3:   Array[0..64] of Char;
    reserved4:   Array[0..64] of Char;
  End;

  TVB_pvfBits = (pvf_b8, pvf_b16, pvf_b32);
  TVB_auBits  = (au_bU, au_b8, au_b16);
  TVB_wavBits = (wav_b8, wav_b16, wav_b32);
  TVB_zyxelComp = (z_2, z_3, z_4);
  TVB_Format    = (fMsg, fAu, fWav, fPvf);

  TVB_SoundFormat = record
   SampleRate: Word;
   Channels:   Byte;
   Case Format: TVB_Format of
      fMsg: (ZyxBits: TVB_zyxelComp);
      fAu: (AuBits: TVB_auBits);
      fWav: (WavBits: TVB_wavBits);
      fPvf: (PvfBits: TVB_pvfBits;
             Ascii: Boolean);
  End;

  TVB_SoundConvert = class(TComponent)
  private
    { Private-Deklarationen }
//    function GetIntputFormat(Stream: TMemoryStream): TVB_SoundFormat;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }

//    function convert_automatic(Stream_in, Stream_out: TMemoryStream; Output_Format: TSoundFormat): Boolean;

    function convert_vbox_to_wav(vbox_Stream, wav_Stream: TMemoryStream): Boolean;
    function convert_wav_to_vbox(wav_Stream, vbox_Stream: TMemoryStream; Header: TVBHeader): Boolean;

    function convert_basic_to_au(basic_Stream, au_Stream: TMemoryStream): Boolean;
    function convert_basic_to_wav(basic_Stream, wav_Stream: TMemoryStream): Boolean;

    function convert_au_to_basic(au_Stream, basic_Stream: TMemoryStream): Boolean;
    function convert_wav_to_basic(wav_Stream, basic_Stream: TMemoryStream): Boolean;

    function convert_basic_to_pvf(basic_Stream, pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
    function convert_au_to_pvf(au_Stream, pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
    function convert_wav_to_pvf(wav_Stream, pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
    function convert_zyxel_to_pvf(zyx_Stream, pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; zyx_comp: TVB_zyxelComp; Ascii: Boolean): Boolean;

    function convert_pvf_to_au(pvf_Stream, au_Stream: TMemoryStream; BitSize: TVB_auBits): Boolean;
    function convert_pvf_to_wav(pvf_Stream, wav_Stream: TMemoryStream; BitSize: TVB_wavBits): Boolean;
    function convert_pvf_to_basic(pvf_Stream, basic_Stream: TMemoryStream): Boolean;
    function convert_pvf_to_adpcm(pvf_Stream, zyxel_Stream: TMemoryStream; BitSize: TVB_zyxelComp): Boolean;

    function check_is_wave_8000_pdc(Stream: TMemoryStream): Boolean;
  published
    { Published-Deklarationen }
  end;

procedure Register;

implementation

uses winsock;

procedure Register;
begin
  RegisterComponents('Beispiele', [TVB_SoundConvert]);
end;

{ TVB_SoundConvert }

//function TSoundConvert.convert_automatic(Stream_in,
//  Stream_out: TMemoryStream; Output_Format: TSoundFormat): Boolean;
//Var
//   InputFormat: TSoundFormat;
//begin
//
//end;

function TVB_SoundConvert.check_is_wave_8000_pdc(Stream: TMemoryStream): Boolean;
Var
   ft: ft_t;
Begin
 Stream.Position := 0;
 ft.fp := Stream;

 Result := wavstartread(ft);

 If Result Then
    If not (ft.info.rate = 8000) Then
       Result := False;
end;

function TVB_SoundConvert.convert_au_to_basic(au_Stream,
  basic_Stream: TMemoryStream): Boolean;
Var
   pvfHeader: Tpvf_header;
   pvf_Stream: TMemoryStream;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := False;
 pvfHeader.Speed := 8000;
 pvfHeader.nBits := 8;

 au_Stream.Position := 0;

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not autopvf(au_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;

 pvfHeader  := Tpvf_header.Create;
 pvf_Stream.Position := 0;
 basic_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftobasic(pvf_Stream,basic_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 pvf_Stream.Free;
 result := True;
end;

function TVB_SoundConvert.convert_au_to_pvf(au_Stream,
  pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
Var
   pvfHeader: Tpvf_header;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := Ascii;
 pvfHeader.Speed := 8000;
 case BitSize of
     pvf_b8:  pvfHeader.nBits := 8;
     pvf_b16: pvfHeader.nBits := 16;
     pvf_b32: pvfHeader.nBits := 32;
 End;

 au_Stream.Position := 0;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not autopvf(au_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_basic_to_au(basic_Stream,
  au_Stream: TMemoryStream): Boolean;
Var
   pvfHeader: Tpvf_header;
   pvf_Stream: TmemoryStream;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := False;
 pvfHeader.Speed := 8000;
 pvfHeader.nBits := 8;

 basic_Stream.Position := 0;

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not basictopvf(basic_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;

 pvfHeader  := Tpvf_header.Create;
 pvf_Stream.Position := 0;
 au_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftoau(pvf_Stream,au_Stream,pvfHeader,SND_FORMAT_MULAW_8) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 pvf_Stream.Free;
 result := True;
end;

function TVB_SoundConvert.convert_basic_to_pvf(basic_Stream, pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
Var
   pvfHeader: Tpvf_header;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := Ascii;
 pvfHeader.Speed := 8000;
 case BitSize of
     pvf_b8:  pvfHeader.nBits := 8;
     pvf_b16: pvfHeader.nBits := 16;
     pvf_b32: pvfHeader.nBits := 32;
 End;
 basic_Stream.Position := 0;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not basictopvf(basic_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_basic_to_wav(basic_Stream,
  wav_Stream: TMemoryStream): Boolean;
Var
   pvfHeader: Tpvf_header;
   pvf_Stream: TmemoryStream;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := False;
 pvfHeader.Speed := 8000;
 pvfHeader.nBits := 8;

 basic_Stream.Position := 0;

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not basictopvf(basic_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;

 pvf_Stream.Position := 0;
 wav_Stream.Clear;

 pvfHeader := Tpvf_Header.Create;
 if not pvfHeader.read_pvf_header(pvf_Stream) then
  Begin
   result := False;
   pvfHeader.Free;
   Exit;
  End;

 if not pvftowav(pvf_Stream,wav_Stream,pvfHeader,8) Then
  Begin
   result := False;
   pvfHeader.Free;
   Exit;
  End;
 pvfHeader.Free;
 pvf_Stream.Free;
 result := True;
end;

function TVB_SoundConvert.convert_pvf_to_adpcm(pvf_Stream,
  zyxel_Stream: TMemoryStream; BitSize: TVB_zyxelComp): Boolean;
Var
   pvfHeader:  Tpvf_header;
   dataFormat: integer;
begin
 dataFormat := 2;
 pvfHeader  := Tpvf_header.Create;
 case BitSize of
     z_2: dataFormat := 2;
     z_3: dataFormat := 3;
     z_4: dataFormat := 4;
 End;

 pvf_Stream.Position := 0;
 zyxel_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftozyxel(pvf_Stream,zyxel_Stream,dataFormat,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_pvf_to_au(pvf_Stream, au_Stream: TMemoryStream; BitSize: TVB_auBits): Boolean;
Var
   pvfHeader:  Tpvf_header;
   dataFormat: integer;
begin
 dataFormat := SND_FORMAT_MULAW_8;
 pvfHeader  := Tpvf_header.Create;
 case BitSize of
     au_bU:  dataFormat := SND_FORMAT_MULAW_8;
     au_b8: dataFormat := SND_FORMAT_LINEAR_8;
     au_b16: dataFormat := SND_FORMAT_LINEAR_16;
 End;

 pvf_Stream.Position := 0;
 au_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftoau(pvf_Stream,au_Stream,pvfHeader,dataFormat) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_pvf_to_basic(pvf_Stream,
  basic_Stream: TMemoryStream): Boolean;
Var
   pvfHeader:  Tpvf_header;
begin
 pvfHeader  := Tpvf_header.Create;

 pvf_Stream.Position := 0;
 basic_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftobasic(pvf_Stream,basic_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_pvf_to_wav(pvf_Stream,
  wav_Stream: TMemoryStream; BitSize: TVB_wavBits): Boolean;
Var
   wav_bits: integer;
   header_in: Tpvf_Header;
begin
 Case BitSize of
     wav_b8:  wav_bits := 8;
     wav_b16: wav_bits := 16;
     wav_b32: wav_bits := 32;
 End;

 pvf_Stream.Position := 0;

 header_in := Tpvf_Header.Create;
 if not header_in.read_pvf_header(pvf_Stream) then
  Begin
   result := False;
   header_in.Free;
   Exit;
  End;

 if not pvftowav(pvf_Stream,wav_Stream,header_in,wav_bits) Then
  Begin
   result := False;
   header_in.Free;
   Exit;
  End;
 header_in.Free;
 result := true;
end;

function TVB_SoundConvert.convert_vbox_to_wav(vbox_Stream,
  wav_Stream: TMemoryStream): Boolean;
Var
   pvf_Stream: TMemoryStream;
   HeaderLessStream: TMemoryStream;
   VBoxHeader: TVBHeader;
begin
 Result := False;

// vbox_Stream.SaveToFile('D:\test.msg');

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 HeaderLessStream := TMemoryStream.Create;
 HeaderLessStream.Clear;

 vbox_Stream.Position := 0;

 vbox_Stream.Read(VBoxHeader,SizeOf(VBoxHeader));
 HeaderLessStream.CopyFrom(vbox_Stream,vbox_Stream.Size - vbox_Stream.Position);

 Case ntohl(VBoxHeader.compression) of
   2: Begin
       If convert_zyxel_to_pvf(HeaderLessStream,pvf_Stream,pvf_b8,z_2,False) Then
          If convert_pvf_to_wav(pvf_Stream,wav_Stream,wav_b8) Then Result := True;
      End;
   3: Begin
       If convert_zyxel_to_pvf(HeaderLessStream,pvf_Stream,pvf_b8,z_3,False) Then
          If convert_pvf_to_wav(pvf_Stream,wav_Stream,wav_b8) Then Result := True;
      End;
   4: Begin
       If convert_zyxel_to_pvf(HeaderLessStream,pvf_Stream,pvf_b8,z_4,False) Then
          If convert_pvf_to_wav(pvf_Stream,wav_Stream,wav_b8) Then Result := True;
      End;
   6: Begin
       If convert_basic_to_wav(HeaderLessStream,wav_Stream) Then Result := True;
      End;
 End;
 pvf_Stream.Free;
end;

function TVB_SoundConvert.convert_wav_to_basic(wav_Stream,
  basic_Stream: TMemoryStream): Boolean;
Var
   pvfHeader: Tpvf_header;
   pvf_Stream: TMemoryStream;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := False;
 pvfHeader.Speed := 8000;
 pvfHeader.nBits := 8;

 wav_Stream.Position := 0;

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not wavtopvf(wav_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;

 pvfHeader  := Tpvf_header.Create;

 pvf_Stream.Position := 0;
 basic_Stream.Clear;

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not pvftobasic(pvf_Stream,basic_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 pvf_Stream.Free;
 result := True;
end;

function TVB_SoundConvert.convert_wav_to_pvf(wav_Stream,
  pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; Ascii: Boolean): Boolean;
Var
   pvfHeader: Tpvf_header;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := Ascii;
 pvfHeader.Speed := 8000;
 case BitSize of
     pvf_b8:  pvfHeader.nBits := 8;
     pvf_b16: pvfHeader.nBits := 16;
     pvf_b32: pvfHeader.nBits := 32;
 End;
 wav_Stream.Position := 0;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not wavtopvf(wav_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;
 result := True;
end;

function TVB_SoundConvert.convert_wav_to_vbox(wav_Stream,
  vbox_Stream: TMemoryStream; Header: TVBHeader): Boolean;
Var
   pvfHeader: Tpvf_header;
   pvf_Stream: TMemoryStream;
begin
 pvfHeader := Tpvf_header.Create;
 pvfHeader.Ascii := False;
 pvfHeader.Speed := 8000;
 pvfHeader.nBits := 8;

 wav_Stream.Position := 0;

 pvf_Stream := TMemoryStream.Create;
 pvf_Stream.Clear;

 if not pvfHeader.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 if not wavtopvf(wav_Stream,pvf_Stream,pvfHeader) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;
 pvfHeader.Free;

 pvfHeader  := Tpvf_header.Create;

 pvf_Stream.Position := 0;
 vbox_Stream.Clear;
 vbox_Stream.Write(Header,SizeOf(Header));

 if not pvfHeader.read_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   pvfHeader.Free;
   Exit
  End;

 Case ntohl(Header.compression) of

      2: Begin
          if not pvftozyxel(pvf_Stream,vbox_Stream,2,pvfHeader) Then
           Begin
            Result := False;
            pvfHeader.Free;
            Exit;
           End;
         End;
      3: Begin
          if not pvftozyxel(pvf_Stream,vbox_Stream,3,pvfHeader) Then
           Begin
            Result := False;
            pvfHeader.Free;
            Exit;
           End;
         End;
      4: Begin
          if not pvftozyxel(pvf_Stream,vbox_Stream,4,pvfHeader) Then
           Begin
            Result := False;
            pvfHeader.Free;
            Exit;
           End;
         End;
      6: Begin
          if not pvftobasic(pvf_Stream,vbox_Stream,pvfHeader) Then
           Begin
            Result := False;
            pvfHeader.Free;
            Exit;
           End;
         End;

 End;

 pvfHeader.Free;
 pvf_Stream.Free;
 result := True;
end;

function TVB_SoundConvert.convert_zyxel_to_pvf(zyx_Stream,
  pvf_Stream: TMemoryStream; BitSize: TVB_pvfBits; zyx_comp: TVB_zyxelComp; Ascii: Boolean): Boolean;
Var
   header_out: Tpvf_header;
   compression: Byte;
begin
 zyx_Stream.Position := 0;
 pvf_Stream.Clear;

 header_out := Tpvf_header.Create;
 header_out.Ascii := Ascii;
 header_out.Speed := 8000;
 header_out.Channels := 1;

 case BitSize of
     pvf_b8:  header_out.nBits  := 8;
     pvf_b16: header_out.nBits  := 16;
     pvf_b32: header_out.nBits  := 32;
 End;

 case zyx_comp of
     z_2: compression  := 2;
     z_3: compression  := 3;
     z_4: compression  := 4;
 End;

 if not header_out.write_pvf_header(pvf_Stream) Then
  Begin
   Result := False;
   header_out.Free;
   Exit
  End;

 result := zyxeltopvf(zyx_Stream, pvf_Stream, compression, header_out);
end;

{
function TVB_SoundConvert.GetIntputFormat(
  Stream: TMemoryStream): TVB_SoundFormat;
Var
   TempStr: String;
   PvfHeader: Tpvf_header;
begin
 Stream.Position := 0;
 Stream.Read(TempStr,4);
 If Pos('PVF',TempStr)  = 1 Then Result.Format := fPvf;
 If Pos('WAVE',TempStr) = 1 Then Result.Format := fWav;
 If Pos('.snd',TempStr) = 1 Then Result.Format := fAu;
 If Pos('VBOX',TempStr) = 1 Then Result.Format := fMsg;

 Stream.Position := 0;

 Case Result.Format of
  fMsg: Exit;
  fAu:  Exit;
  fWav: Exit;
  fPvf: Begin
         PvfHeader := Tpvf_header.Create;
         PvfHeader.read_pvf_header(Stream);
         Result.SampleRate := PvfHeader.Speed;
         Result.Channels   := PvfHeader.Channels;
         Result.Ascii      := PvfHeader.Ascii;
         Case PvfHeader.nBits of
             8:  Result.PvfBits := pvf_b8;
             16: Result.PvfBits := pvf_b16;
             32: Result.PvfBits := pvf_b32;
         End;
         PvfHeader.Free;
        End;
 End;
end;
}
end.
