unit vb_wav;

interface

uses classes, SysUtils, vb_pvflib, Windows;

const

     WAVE_FORMAT_UNKNOWN   = $0000;
     WAVE_FORMAT_PCM       = $0001;
     WAVE_FORMAT_ADPCM     = $0002;
     WAVE_FORMAT_ALAW      = $0006;
     WAVE_FORMAT_MULAW     = $0007;
     WAVE_FORMAT_OKI_ADPCM = $0010;
     WAVE_FORMAT_DIGISTD   = $0015;
     WAVE_FORMAT_DIGIFIX   = $0016;
     IBM_FORMAT_MULAW      = $0101;
     IBM_FORMAT_ALAW       = $0102;
     IBM_FORMAT_ADPCM      = $0103;

     //* flags field */
     FILE_STEREO = 1;    //* does file format support stereo? */
     FILE_LOOPS  = 2;    //* does file format support loops? */
     FILE_INSTR  = 4;    //* does file format support instrument specificications? */

     //* Size field */
     S_BYTE   = 1;
     S_WORD   = 2;
     S_LONG   = 4;
     S_FLOAT  = 5;
     S_DOUBLE = 6;
     S_IEEE   = 7;         //* IEEE 80-bit floats.  Is it necessary? */

     //* Style field */
     UNSIGNED    = 1;    //* unsigned linear: Sound Blaster */
     SIGN2       = 2;    //* signed linear 2's comp: Mac */
     ULAW        = 3;    //* U-law signed logs: US telephony, SPARC */
     ALAW        = 4;    //* A-law signed logs: non-US telephony */


     sizes: Array[0..7] of String = (
                                     'NONSENSE!',
                                     'bytes',
                                     'shorts',
                                     'NONSENSE',
                                     'longs',
                                     '32-bit floats',
                                     '64-bit floats',
                                     'IEEE floats'
                                    );

Type
//* Private data for .wav file */

    wav_t = record
     samples:       LongInt;
     second_header: Integer; //* non-zero on second header write */
    End;

//* Signal parameters */

    signalinfo = record
     rate:     LongInt;         //* sampling rate */
     size:     integer;         //* word length of data */
     style:    integer;         //* format of sample numbers */
     channels: integer;         //* number of sound channels */
    End;

//* Loop parameters */

    loopinfo = record
     start:  integer;         //* first sample */
     length: integer;         //* length */
     count:  integer;         //* number of repeats, 0=forever */
     ltype:  integer;         //* 0=no, 1=forward, 2=forward/back */
    End;

//* Instrument parameters */

//* vague attempt at generic information for sampler-specific info */

    instrinfo = record
     MIDInote: Byte;  //* for unity pitch playback */
     MIDIlow:  Byte;
     MIDIhi:   Byte;  //* MIDI pitch-bend range */
     loopmode: Byte;  //* semantics of loop data */
     nloops:   Byte;  //* number of active loops */
     smpte:    Array[0..3] of Byte; //* SMPTE offset (hour:min:sec:frame) */
                                    //* this is a film audio thing */
    End;

 TWavHeader = packed record
   magic: Array[0..3] of Char;
 End;

 TWavChunk = packed record
   magic: Array[0..3] of Char;
   size:  LongWord;
 End;

 TWavFormatChunk = packed record
   wFormatTag: WORD;         // Format category
   wChannels: WORD;          // Number of channels
   dwSamplesPerSec: DWORD;    // Sampling rate
   dwAvgBytesPerSec: DWORD;   // For buffer estimation
   wBlockAlign: WORD;        // Data block size
 End;

 TWavPCMspec = packed record
   wBitsPerSample: WORD;
 End;


const
     MIDI_UNITY = 60;         //* MIDI note number to play sample at unity */

     //* Loop modes */
     LOOP_NONE          = 0;
     LOOP_8             = 1;  //* 8 loops: don't know ?? */
     LOOP_SUSTAIN_DECAY = 2;  //* AIFF style: one sustain & one decay loop */

(*
 *  Format information for input and output files.
 *)

     PRIVSIZE  = 100;

     NLOOPS    =   8;


Type
    soundstream = record
     info: signalinfo;                      //* signal specifications */
     instr: instrinfo;                      //* instrument specification */
     loops: Array[0..NLOOPS-1] of loopinfo; //* Looping specification */
     swap: Byte;                            //* do byte- or word-swap */
     seekable: Byte;                        //* can seek on this file */
     filename: String;                      //* file name */
     filetype: String;                      //* type of file */
     comment: String;                       //* comment string */
     fp: TMemoryStream;                     //* File stream pointer */
//     format_t *h;             /* format struct for this file */
     priv: Array[0..(PRIVSIZE div 8)-1] of double;   //* format's private data area */
    End;

    ft_t = soundstream;

//* wave file characteristics */

Var
   wFormatTag: Word;             //* data format */
   wChannels: Word;              //* number of channels */
   wSamplesPerSecond: LongWord;  //* samples per second per channel */
   wAvgBytesPerSec: LongWord;    //* estimate of bytes per second needed */
   wBlockAlign: Word;            //* byte alignment of a basic sample block */
   wBitsPerSample: Word;         //* bits per sample */
   data_length: LongWord;        //* length of sound data in bytes */
   bytespersample: LongWord;     //* bytes per sample (per channel) */


function readChunk(ft: ft_t; var nextPos: Integer): TWavChunk;
function checkRIFF(ft: ft_t): Boolean;
function checkWAVE(ft: ft_t):Boolean;
procedure readFMT(ft: ft_t; var WavFormatChunk: TWavFormatChunk; var WavPCMSpecial:  TWavPCMspec);
procedure skipToNextChunk(ft: ft_t; nextPos: Integer);

function rlshort(ft: ft_t): Word;
function rllong(ft: ft_t): LongWord;
function wllong(ft: ft_t; ul: LongWord): Boolean;
function wlshort(ft: ft_t;us: word): Boolean;
function wavstartread(var ft: ft_t): Boolean;
function wavwritehdr(var ft: ft_t;data_size: LongInt): Boolean;
function wavstartwrite(var ft: ft_t;data_size: LongInt): Boolean;
function pvftowav(fd_in, fd_out: TMemoryStream; header_in: Tpvf_header; wav_bits: integer): Boolean;
function wavtopvf(fd_in, fd_out: TMemoryStream; header_out: Tpvf_header): Boolean;

implementation

function readChunk(ft: ft_t; var nextPos: Integer): TWavChunk;
begin
 FillChar(Result,SizeOf(Result),#0);
 ft.fp.Read(Result,SizeOf(Result));
 nextPos := ft.fp.Position + Result.size;
end;

function checkRIFF(ft: ft_t): Boolean;
Var
   WavRIFF: TWavChunk;
Begin
 ft.fp.Position := 0;
 FillChar(WavRIFF,SizeOf(WavRIFF),#0);
 ft.fp.Read(WavRIFF,SizeOf(WavRIFF));
 Result := (((SizeOf(WavRIFF) + WavRIFF.size) = ft.fp.Size) and
           (WavRIFF.magic = 'RIFF'));
End;

function checkWAVE(ft: ft_t):Boolean;
Var
   WavHeader:      TWavHeader;
Begin
 FillChar(WavHeader,SizeOf(WavHeader),#0);
 ft.fp.Read(WavHeader,SizeOf(WavHeader));
 Result := (WavHeader.magic = 'WAVE');
End;

procedure readFMT(ft: ft_t; var WavFormatChunk: TWavFormatChunk; var WavPCMSpecial:  TWavPCMspec);
begin
 FillChar(WavFormatChunk,SizeOf(WavFormatChunk),#0);
 FillChar(WavPCMSpecial,SizeOf(WavPCMSpecial),#0);

 ft.fp.Read(WavFormatChunk,SizeOf(WavFormatChunk));
 if WavFormatChunk.wFormatTag = WAVE_FORMAT_PCM Then ft.fp.Read(WavPCMSpecial,SizeOf(WavPCMSpecial));
end;

procedure skipToNextChunk(ft: ft_t; nextPos: Integer);
begin
 ft.fp.Position := nextPos;
end;

function rlshort(ft: ft_t): Word;
Var
   Byte1,
   Byte2: Byte;
Begin
 ft.fp.Read(Byte1,1);
 ft.fp.Read(Byte2,1);
 result := (Byte2 shl 8) or Byte1;
End;

function rllong(ft: ft_t): LongWord;
Var
   data:   array[0..3] of Byte;
   MyWord: LongInt;
   MyChar: Char;
   Count:  Integer;
Begin
 For Count := 0 To 3 Do
  Begin
   ft.fp.Read(MyChar,1);
   data[count] := ord(MyChar);
  End;
 MyWord := data[3];
 MyWord := (MyWord shr 8) or data[2];
 MyWord := (MyWord shr 16) or data[1];
 MyWord := (MyWord shr 24) or data[0];
 result := MyWord;
End;
(*Var
   Byte1,
   Byte2,
   Byte3,
   Byte4: Byte;
   MyLong: LongWord;
Begin
// ft.fp.Read(Byte1,1);
// ft.fp.Read(Byte2,1);
// ft.fp.Read(Byte3,1);
// ft.fp.Read(Byte4,1);
// result := ((Byte4 shl 24) or (Byte3 shl 16) or (Byte2 shl 8) or (Byte1))
 ft.fp.Read(MyLong,4);
 result := not MyLong;
End;
*)
function wllong(ft: ft_t; ul: LongWord): Boolean;
Var
   data: Byte;
Begin
 result := True;
 Try
  data := ul and $ff;
  ft.fp.Write(data,1);
  data := (ul shr 8) and $ff;
  ft.fp.Write(data,1);
  data := (ul shr 16) and $ff;
  ft.fp.Write(data,1);
  data := (ul shr 24) and $ff;
  ft.fp.Write(data,1);
 except
  result := False;
 End;
End;

function wlshort(ft: ft_t; us: word): Boolean;
Var
   Data: ShortInt;
Begin
 result := true;
 Try
  Data := ShortInt(us);
  ft.fp.Write(Data,1);
  Data := ShortInt(us shr 8);
  ft.fp.Write(Data,1);
 except
  result := false;
 End;
End;

function wavstartread(var ft: ft_t): Boolean;
Var
   wav: wav_t;
   WavRIFF,
   WavChunk,
   WavDataChunk:   TWavChunk;
   WavHeader:      TWavHeader;
   WavFormatChunk: TWavFormatChunk;
   WavPCMSpecial:  TWavPCMspec;
   nextPos:        Integer;
Begin
 FillChar(wav,SizeOf(wav),#0);
 FillChar(WavRIFF,SizeOf(WavRIFF),#0);
 FillChar(WavChunk,SizeOf(WavChunk),#0);
 FillChar(WavDataChunk,SizeOf(WavDataChunk),#0);
 FillChar(WavHeader,SizeOf(WavHeader),#0);
 FillChar(WavFormatChunk,SizeOf(WavFormatChunk),#0);
 FillChar(WavPCMSpecial,SizeOf(WavPCMSpecial),#0);

 ft.info.rate      := 0;
 ft.info.size      := -1;
 ft.info.style     := -1;
 ft.info.channels  := -1;
 ft.comment        := '';
 ft.swap           := 0;

 if not checkRIFF(ft) Then
  Begin
   result := false;
   Exit;
  End;

 if not checkWAVE(ft) Then
  Begin
   result := false;
   Exit;
  End;

 WavChunk := readChunk(ft,nextPos);
 if not ((WavChunk.magic = 'fmt ') and (WavChunk.size >= 16)) Then //Sorry, fmt chunk not found or too short
  begin
   result := false;
   Exit;
  end;

 readFMT(ft,WavFormatChunk,WavPCMSpecial);
 skipToNextChunk(ft,nextPos);

 Repeat
  Begin
   WavChunk := readChunk(ft,nextPos);
   if (WavChunk.magic <> 'data') Then
      skipToNextChunk(ft,nextPos);
  End;
 Until (WavChunk.magic = 'data');

 if (WavChunk.magic = 'data') Then
  Begin
   WavDataChunk.magic := WavChunk.magic;
   WavDataChunk.size  := WavChunk.size;
  End;

 if ((WavFormatChunk.wChannels < 1) and (CompareStr('data',WavChunk.magic) <> 0)) Then   //* "fmt " chunk or data chunk not found */
  Begin
   result := false;
   Exit;
  End;

  Case WavFormatChunk.wFormatTag of
          WAVE_FORMAT_UNKNOWN:    Begin
                                   result := false;
                                   Exit;
                                  End;
          WAVE_FORMAT_PCM:        Begin //* this one, at least, I can handle */

                                  End;
          WAVE_FORMAT_ADPCM:      Begin
                                   result := false;
                                   Exit;
                                  End;  //"Sorry, this WAV file is in Microsoft ADPCM format."
          WAVE_FORMAT_ALAW:       ft.info.style := ALAW; //* Think I can handle this */
          WAVE_FORMAT_MULAW:      ft.info.style := ULAW; //* Think I can handle this */
          WAVE_FORMAT_OKI_ADPCM:  Begin
                                   result := false;
                                   Exit;
                                  End; //"Sorry, this WAV file is in OKI ADPCM format."
          WAVE_FORMAT_DIGISTD:    Begin
                                   result := false;
                                   Exit;
                                  End; //"Sorry, this WAV file is in Digistd format."
          WAVE_FORMAT_DIGIFIX:    Begin
                                   result := false;
                                   Exit;
                                  End; //Sorry, this WAV file is in Digifix format."
          IBM_FORMAT_MULAW:       Begin
                                   result := false;
                                   Exit;
                                  End; //Sorry, this WAV file is in IBM U-law format."
          IBM_FORMAT_ALAW:        Begin
                                   result := false;
                                   Exit;
                                  End; //Sorry, this WAV file is in IBM A-law format."
          IBM_FORMAT_ADPCM:       Begin
                                   result := false;
                                   Exit;
                                  End; //Sorry, this WAV file is in IBM ADPCM format."
          Else Begin
                result := false;
                Exit;
               End; //"Sorry, don't understand format"
  End;

  wChannels         := WavFormatChunk.wChannels;
  ft.info.channels  := WavFormatChunk.wChannels;
  wSamplesPerSecond := WavFormatChunk.dwSamplesPerSec;
  ft.info.rate      := WavFormatChunk.dwSamplesPerSec;
  wAvgBytesPerSec   := WavFormatChunk.dwAvgBytesPerSec; //* Average bytes/second */
  wBlockAlign       := WavFormatChunk.wBlockAlign;    //* Block align */
  bytespersample    := WavFormatChunk.wBlockAlign;
  if WavFormatChunk.wFormatTag = WAVE_FORMAT_PCM Then wBitsPerSample := WavPCMSpecial.wBitsPerSample
                                                 Else wBitsPerSample := WavFormatChunk.wBlockAlign * 8;     //* bits per sample per channel */

  Case WavFormatChunk.wBlockAlign of
          1: ft.info.size := S_BYTE;
          2: ft.info.size := S_WORD;
          4: ft.info.size := S_LONG;
          Else Begin
                result := false;
                Exit;
               End; // "Sorry, don't understand .wav size"
  End;

  data_length := WavChunk.size;
  wav.samples := data_length div ft.info.size;    //* total samples */

  result := true;
End;

function wavwritehdr(var ft: ft_t;data_size: LongInt): Boolean;
Var
   WavRIFF,
   WavChunk,
   WavDataChunk:   TWavChunk;
   WavHeader:      TWavHeader;
   WavFormatChunk: TWavFormatChunk;
   WavPCMSpecial:  TWavPCMspec;
Begin
 FillChar(WavRIFF,SizeOf(WavRIFF),#0);
 FillChar(WavChunk,SizeOf(WavChunk),#0);
 FillChar(WavDataChunk,SizeOf(WavDataChunk),#0);
 FillChar(WavHeader,SizeOf(WavHeader),#0);
 FillChar(WavFormatChunk,SizeOf(WavFormatChunk),#0);
 FillChar(WavPCMSpecial,SizeOf(WavPCMSpecial),#0);

 Case ft.info.size of
          S_BYTE: Begin
                   wBitsPerSample := 8;
                   if (ft.info.style = -1) or (ft.info.style = UNSIGNED) then ft.info.style := UNSIGNED;
                  End;
          S_WORD: Begin
                   wBitsPerSample := 16;
                   if (ft.info.style = -1) or (ft.info.style = SIGN2) then ft.info.style := SIGN2;
                  End;
          S_LONG: Begin
                   wBitsPerSample := 32;
                   if (ft.info.style = -1) or (ft.info.style = SIGN2) then ft.info.style := SIGN2;
                  End;
          Else    Begin
                   wBitsPerSample := 32;
                   if (ft.info.style = -1) then ft.info.style := SIGN2;
                  End;
 End;

 Case ft.info.style of
          UNSIGNED: wFormatTag := WAVE_FORMAT_PCM;
          SIGN2:    wFormatTag := WAVE_FORMAT_PCM;
          ALAW:     wFormatTag := WAVE_FORMAT_ALAW;
          ULAW:     wFormatTag := WAVE_FORMAT_MULAW;
 End;

 wSamplesPerSecond := ft.info.rate;
 bytespersample    := (wBitsPerSample + 7) div 8;
 wAvgBytesPerSec   := ft.info.rate * ft.info.channels * bytespersample;
 wChannels         := ft.info.channels;
 wBlockAlign       := ft.info.channels * bytespersample;
 data_length       := data_size;

 //* figured out header info, so write it */

 WavRIFF.magic := 'RIFF';
 WavRIFF.size  := SizeOf(WavHeader) + SizeOf(WavChunk) + SizeOf(WavFormatChunk) + SizeOf(WavPCMSpecial) + SizeOf(WavChunk);

 WavHeader.magic := 'WAVE';

 WavChunk.magic  := 'fmt ';
 WavChunk.size   := SizeOf(WavFormatChunk) + SizeOf(WavPCMSpecial);

 WavFormatChunk.wFormatTag       := wFormatTag;
 WavFormatChunk.wChannels        := wChannels;
 WavFormatChunk.dwSamplesPerSec  := wAvgBytesPerSec;
 WavFormatChunk.dwAvgBytesPerSec := wAvgBytesPerSec;
 WavFormatChunk.wBlockAlign      := wBlockAlign;

 WavPCMSpecial.wBitsPerSample    := wBitsPerSample;

 WavDataChunk.magic  := 'data';
 WavDataChunk.size   := data_length;

 ft.fp.Write(WavRIFF,SizeOf(WavRIFF));
 ft.fp.Write(WavHeader,SizeOf(WavHeader));
 ft.fp.Write(WavChunk,SizeOf(WavChunk));
 ft.fp.Write(WavFormatChunk,SizeOf(WavFormatChunk));
 ft.fp.Write(WavPCMSpecial,SizeOf(WavPCMSpecial));
 ft.fp.Write(WavDataChunk,SizeOf(WavDataChunk));

 result := true;
End;

function wavstartwrite(var ft: ft_t;data_size: LongInt): Boolean;
Var
   wav: wav_t;
Begin
 FillChar(wav,SizeOf(wav),#0);
 ft.swap := 0;

 wav.samples := 0;
 wav.second_header := 0;

 result := wavwritehdr(ft,data_size);
End;

function pvftowav(fd_in, fd_out: TMemoryStream; header_in: Tpvf_header; wav_bits: integer): Boolean;
Var
   bytespersample: integer;
//   data_size: LongInt;
//   buffer_size: integer;
//   voice_samples: integer;
   s: soundstream;
   Stream_data: TMemoryStream;
   sample:      integer;
   MyShort:     ShortInt;
Begin
 bytespersample := (wav_bits + 7) div 8;

 Case bytespersample of
          1: s.info.size := S_BYTE;
          2: s.info.size := S_WORD;
          4: s.info.size := S_LONG;
          Else Begin
                result := false; //"sorry, don't understand .wav size"
                exit;
               End;
  End;

 s.info.rate     := header_in.speed;
 s.info.style    := -1;
 s.info.channels := header_in.channels;
 s.comment       := '';
 s.swap          := 0;
 s.filetype      := '0';
 s.fp            := fd_out;
 s.seekable      := 0;

 Stream_data := TMemoryStream.Create;
 Case s.info.size of
          S_BYTE: Begin
                   While fd_in.Position < fd_in.Size Do
                    Begin
                     sample := ShortInt(header_in.read_pvf_data(fd_in) shr 16);
                     sample := ShortInt(sample + $80);
                     Stream_data.Write(sample,1);
                    End;
                  End;
          S_WORD: Begin
                   s.info.rate := s.info.rate Div 2;
                   While fd_in.Position < fd_in.Size Do
                    Begin
                     sample := SmallInt(header_in.read_pvf_data(fd_in) shr 8);
                     MyShort := ShortInt(sample);
                     Stream_data.Write(MyShort,1);
                     MyShort := ShortInt(sample shr 8);
                     Stream_data.Write(MyShort,1);
                    End;
                  End;
          S_LONG: Begin
  { TODO : get 32bit working }
                   While fd_in.Position < fd_in.Size Do
                    Begin
                     sample := header_in.read_pvf_data(fd_in);
                     MyShort := sample;
                     Stream_data.Write(MyShort,1);
                     MyShort := sample shr 8;
                     Stream_data.Write(MyShort,1);
                     MyShort := sample shr 16;
                     Stream_data.Write(MyShort,1);
                     MyShort := sample shr 24;
                     Stream_data.Write(MyShort,1);
                    End;
                  End;
 End;

 If not wavstartwrite(s,Stream_data.Size) then
  Begin
   result := false;
   Exit;
  End;
 Stream_data.Position := 0;
 s.fp.CopyFrom(Stream_data,0);
 result := true;
End;

function wavtopvf(fd_in, fd_out: TMemoryStream; header_out: Tpvf_header): Boolean;
Var
   s: soundstream;
   data:      integer;
   MyChar:    Char;
Begin
 s.fp := fd_in;

 if not wavstartread(s) then
  Begin
   result := false;
   Exit;
  End;

 header_out.channels := wChannels;
 header_out.speed    := wSamplesPerSecond;
 header_out.write_pvf_header(fd_out);

 While s.fp.Position < s.fp.Size Do
  Begin
   Case wBitsPerSample of
        8:   Begin
              s.fp.Read(MyChar,1);
              data := ord(MyChar);
              data := data - $80;
              data := data shl 16;
             End;
        16:  Begin
              s.fp.Read(MyChar,1);
              data := ord(MyChar);
              data := data and $ff;
              s.fp.Read(MyChar,1);
              data := data + (ord(MyChar) shr 8);
              if ((data and $8000) = 1) then data := data or $FFFF0000;
              data := data shl 8;
             End;
        Else Begin             // "unsupported number of bits per sample"
              result := false;
              exit;
             End;
   End;
   header_out.write_pvf_data(fd_out,data);
  End;
 result := true;
End;


end.
